%  Import Volocity - Item as TIFF (*.tif;*.tiff) Function
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory.
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="ImportVolocityTif" icon="Matlab" tooltip="Import a file exported from Volocity as : Item as TIFF (*.tif;*.tiff) file.">
%          <Command>MatlabXT::ImportVolocityTif(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%
%  Description:
%
%   Reduce the number of time points by deleting inbetween time points.
%

function ImportVolocityTif(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end

% get the data set
cp = cd();
[filename,path] = uigetfile('*.tif');
if path ~= 0
    cd(path);
    info = imfinfo(filename);
    cd(cp);
end

% Find the size of dataset
str = info(1).ImageDescription;
numDEF = {'TotalZPlanes=','TotalChannels=','TotalTimepoints='};
numValues = zeros(size(numDEF,2),1);
for i = 1:size(numDEF,2)
    pos = strfind(str,numDEF{i});
    numValues(i) = sscanf(str(pos:end),strcat(numDEF{i},'%f,%f,%f'));
end
vNumberOfSlices = numValues(1);
vNumberOfChannels = numValues(2);
vNumberOfTimePoints = numValues(3);

switch info(1).BitDepth
    case 8
        aType = Imaris.tType.eTypeUInt8;
    case 16
        aType = Imaris.tType.eTypeUInt16;
    case 32
        aType = Imaris.tType.eTypeFloat;
end

% prepare the dataset
vDataSet = vImarisApplication.GetFactory.CreateDataSet;
vDataSet.Create(aType, info(1).Width, info(1).Height, ...
    vNumberOfSlices, vNumberOfChannels, vNumberOfTimePoints);
ColorSet = false(vNumberOfChannels,1);
vTimePointsDelta = zeros(vNumberOfTimePoints,1);

hw = waitbar(0,'Creating new data set');
cd(path);
for datasetId = 1:size(info,1)
    str = info(datasetId).ImageDescription;
    numDEF = {'ZPlane=','ChannelNo=','TimepointNo=','TimeStampMicroSeconds='};
    numValues = zeros(size(numDEF,2),1);
    for i = 1:size(numDEF,2)
        pos = strfind(str,numDEF{i});
        numValues(i) = sscanf(str(pos:end),strcat(numDEF{i},'%f'));
    end
    vSlice = numValues(1);
    vChannelIndex = numValues(2);
    vTime = numValues(3);
    vTimeStamp = numValues(4);
    % read slice data
    vData = imread(filename,datasetId);
    if strcmp(vDataSet.GetType,'eTypeUInt8')
        vDataSet.SetDataSliceBytes(vData', vSlice-1, vChannelIndex-1, vTime-1);
    elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
        vDataSet.SetDataSliceShorts(vData', vSlice-1, vChannelIndex-1, vTime-1);
    elseif strcmp(vDataSet.GetType, 'eTypeFloat')
        vDataSet.SetDataSliceFloats(vData', vSlice-1, vChannelIndex-1, vTime-1);
    end
    if ~ColorSet(vChannelIndex)
        % find the Channel color
        pos = strfind(str,'WhiteColour=');
        ColorRGB = sscanf(str(pos:end),strcat('WhiteColour=','%f,%f,%f'));
        ColorRGBA = rgb2rgba(ColorRGB(1),ColorRGB(2),ColorRGB(3));
        vDataSet.SetChannelColorRGBA(vChannelIndex-1, ColorRGBA);
        ColorSet(vChannelIndex) = true;
    end
    if vChannelIndex == 1
        if vTimeStamp == 0
            aTimePoint = datevec(now());
        else
            aTimePoint = datevec(vTimeStamp/1000000/3600/24);
            aTimePoint(1) = aTimePoint(1) + 1904; % TimeStamp starts from 1904
            aTimePoint(4) = aTimePoint(4) - 5; % with 5 hours before
        end
        vTimePointsDelta(vTime) = datenum(aTimePoint);
        aTimePoint = datestr(aTimePoint,'yyyy-mm-dd HH:MM:SS.FFF');
        vDataSet.SetTimePoint(vTime-1,aTimePoint);
        if vTime > 1
            vDelta = (vTimePointsDelta(vTime) - vTimePointsDelta(vTime-1))*24*3600;
            vDataSet.SetTimePointsDelta(vDelta);
        end
    end
    
    waitbar(datasetId/size(info,1),hw)
end
cd(cp);

% Find the voxel size
str = info(1).ImageDescription;
numDEF = {'XCalibrationMicrons=','YCalibrationMicrons=',...
    'ZCalibrationMicrons=','XLocationMicrons=',...
    'YLocationMicrons=','ZLocationMicrons='};
numValues = zeros(size(numDEF,2),1);
numValues(1:3) = 1;
for i = 1:size(numDEF,2)
    pos = strfind(str,numDEF{i});
    if ~isempty(pos)
        numValues(i) = sscanf(str(pos:end),strcat(numDEF{i},'%f'));
    end
end
dX = numValues(1);
dY = numValues(2);
dZ = numValues(3);
Xmin = numValues(4);
Ymin = numValues(5);
Zmin = numValues(6);
Xmax = Xmin+info(1).Width*dX;
Ymax = Ymin+info(1).Height*dY;
Zmax = Zmin+vNumberOfSlices*dZ;

% set ExtendMaxXYZ
vDataSet.SetExtendMinX(Xmin);
vDataSet.SetExtendMinY(Ymin);
vDataSet.SetExtendMinZ(Zmin);
vDataSet.SetExtendMaxX(Xmax);
vDataSet.SetExtendMaxY(Ymax);
vDataSet.SetExtendMaxZ(Zmax);

close(hw);

vImarisApplication.SetDataSet(vDataSet);
vImarisApplication.GetSurpassCamera.Fit;

function rgba = rgb2rgba(R,G,B)

rgba = bin2dec(sprintf('%08d%08d%08d',str2double(dec2bin(B)),...
    str2double(dec2bin(G)),str2double(dec2bin(R))));
